""" Common functions for package biobb_analysis.gromacs """
import os.path
from biobb_common.tools import file_utils as fu


def check_energy_path(path, obj):
	""" Checks energy input file """ 
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(path):
		fu.log('Unexisting energy input file, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting energy input file')
	filename, file_extension = os.path.splitext(path)
	if not is_valid_energy(file_extension[1:]):
		fu.log('Format %s in energy input file is not compatible' % file_extension[1:], out_log, obj.global_log)
		raise SystemExit('Format %s in energy input file is not compatible' % file_extension[1:])
	return path

def check_input_path(path, obj):
	""" Checks input structure file """ 
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(path):
		fu.log('Unexisting structure input file, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting structure input file')
	filename, file_extension = os.path.splitext(path)
	if not is_valid_structure(file_extension[1:]):
		fu.log('Format %s in structure input file is not compatible' % file_extension[1:], out_log, obj.global_log)
		raise SystemExit('Format %s in structure input file is not compatible' % file_extension[1:])
	return path

def check_traj_path(path, obj):
	""" Checks input structure file """ 
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(path):
		fu.log('Unexisting trajectory input file, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting trajectory input file')
	filename, file_extension = os.path.splitext(path)
	if not is_valid_trajectory(file_extension[1:]):
		fu.log('Format %s in trajectory input file is not compatible' % file_extension[1:], out_log, obj.global_log)
		raise SystemExit('Format %s in trajectory input file is not compatible' % file_extension[1:])
	return path

def check_out_xvg_path(path, obj):
	""" Checks if output folder exists and format is xvg """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(os.path.dirname(path)):
		fu.log('Unexisting output folder, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting output folder')
	filename, file_extension = os.path.splitext(path)
	if not is_valid_xvg(file_extension[1:]):
		fu.log('Format %s in output file is not compatible' % file_extension[1:], out_log, obj.global_log)
		raise SystemExit('Format %s in output file is not compatible' % file_extension[1:])
	return path

def check_out_pdb_path(path, obj):
	""" Checks if output folder exists and format is xvg """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(os.path.dirname(path)):
		fu.log('Unexisting output folder, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting output folder')
	filename, file_extension = os.path.splitext(path)
	if not is_valid_structure(file_extension[1:]):
		fu.log('Format %s in output file is not compatible' % file_extension[1:], out_log, obj.global_log)
		raise SystemExit('Format %s in output file is not compatible' % file_extension[1:])
	return path

def check_conf(path):
	""" Checks configuration file """
	if not os.path.exists(path):
		raise SystemExit('Unexisting configuration file')
	return path

def get_default_value(key):
	""" Gives default values according to the given key """
	default_values = {
		"instructions_file": "instructions.in",
		"gmx_path": "gmx",
		"terms": ["Potential"],
		"selection": "Protein-H"
	}

	return default_values[key]

def get_binary_path(properties, type):
	""" Gets binary path """
	return properties.get(type, get_default_value(type))

def get_terms(properties, obj):
	""" Gets energy terms """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	terms = properties.get('terms', dict())
	if not terms or not isinstance(terms, list):
		fu.log('No terms provided or incorrect format, exiting', out_log, obj.global_log)
		raise SystemExit('No terms provided or incorrect format')
	if not is_valid_term(terms):
		fu.log('Incorrect terms provided, exiting', out_log, obj.global_log)
		raise SystemExit('Incorrect terms provided')
	return properties.get('terms', '')

def get_selection(properties, obj):
	""" Gets selection items """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	selection = properties.get('selection', get_default_value('selection'))
	if not selection:
		fu.log('No selection provided or incorrect format, exiting', out_log, obj.global_log)
		raise SystemExit('No selection provided or incorrect format')
	if not is_valid_selection(selection):
		fu.log('Incorrect selection provided, exiting', out_log, obj.global_log)
		raise SystemExit('Incorrect selection provided')
	return selection

def get_xvg(properties, obj):
	""" Gets xvg """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	xvg = properties.get('xvg', 'none')
	if not is_valid_xvg_param(xvg):
		fu.log('Incorrect xvg provided, exiting', out_log, obj.global_log)
		raise SystemExit('Incorrect xvg provided')
	return xvg

def is_valid_structure(ext):
	""" Checks if structure format is compatible with GROMACS """
	formats = ['tpr', 'gro', 'g96', 'pdb', 'brk', 'ent']
	return ext in formats

def is_valid_trajectory(ext):
	""" Checks if trajectory format is compatible with GROMACS """
	formats = ['xtc', 'trr', 'cpt', 'gro', 'g96', 'pdb', 'tng']
	return ext in formats

def is_valid_energy(ext):
	""" Checks if energy format is compatible with GROMACS """
	formats = ['edr']
	return ext in formats

def is_valid_xvg(ext):
	""" Checks if file is XVG """
	formats = ['xvg']
	return ext in formats

def is_valid_xvg_param(ext):
	""" Checks xvg parameter """
	formats = ['xmgrace', 'xmgr', 'none']
	return ext in formats

def is_valid_term(iterms):
	""" Checks if term is correct """
	cterms = ['Angle', 'Proper-Dih.', 'Improper-Dih.', 'LJ-14', 'Coulomb-14', 'LJ-(SR)', 'Coulomb-(SR)', 'Coul.-recip.', 'Position-Rest.', 'Potential', 'Kinetic-En.', 'Total-Energy', 'Temperature', 'Pressure', ' Constr.-rmsd', 'Box-X', 'Box-Y', ' Box-Z', 'Volume', 'Density', 'pV', 'Enthalpy', 'Vir-XX', 'Vir-XY', 'Vir-XZ', 'Vir-YX', 'Vir-YY', 'Vir-YZ', 'Vir-ZX', 'Vir-ZY', 'Vir-ZZ', 'Pres-XX', 'Pres-XY', 'Pres-XZ', 'Pres-YX', 'Pres-YY', 'Pres-YZ', 'Pres-ZX', 'Pres-ZY', 'Pres-ZZ', '#Surf*SurfTen', 'Box-Vel-XX', 'Box-Vel-YY', 'Box-Vel-ZZ', 'Mu-X', 'Mu-Y', 'Mu-Z', 'T-Protein', 'T-non-Protein', 'Lamb-Protein', 'Lamb-non-Protein']
	return all(elem in cterms for elem in iterms)

def is_valid_selection(ext):
	""" Checks if selection is correct """
	formats = ['System', 'Protein', 'Protein-H', 'C-alpha', 'Backbone', 'MainChain', 'MainChain+Cb', 'MainChain+H', 'SideChain', 'SideChain-H', 'Prot-Masses', 'non-Protein', 'Water', 'SOL', 'non-Water', 'Ion', 'NA', 'CL', 'Water_and_ions']
	return ext in formats



""" Common functions for package biobb_analysis.ambertools """
import os.path
from biobb_common.tools import file_utils as fu


def check_top_path(path, obj):
	""" Checks topology input file """ 
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(path):
		fu.log('Unexisting topology input file, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting topology input file')
	filename, file_extension = os.path.splitext(path)
	if not is_valid_topology(file_extension[1:]):
		fu.log('Format %s in topology input file is not compatible' % file_extension[1:], out_log, obj.global_log)
		raise SystemExit('Format %s in topology input file is not compatible' % file_extension[1:])
	return path

def check_traj_path(path, obj):
	""" Checks trajectory input file """ 
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(path):
		fu.log('Unexisting trajectory input file, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting trajectory input file')
	filename, file_extension = os.path.splitext(path)
	if not is_valid_trajectory(file_extension[1:]):
		fu.log('Format %s in trajectory input file is not compatible' % file_extension[1:], out_log, obj.global_log)
		raise SystemExit('Format %s in trajectory input file is not compatible' % file_extension[1:])
	return path

def check_out_path(path, obj):
	""" Checks if output folder exists """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(os.path.dirname(path)):
		fu.log('Unexisting output folder, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting output folder')
	return path

def check_conf(path):
	""" Checks configuration file """
	if not os.path.exists(path):
		raise SystemExit('Unexisting configuration file')
	return path

def get_parameters(properties, type, obj):
	""" Gets in_parameters and out_parameters """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not properties.get(type, dict()):
		fu.log('No %s parameters provided' % type, out_log, obj.global_log)
		raise SystemExit('No %s parameters provided' % type)
	else:
		return {k: v for k, v in properties.get(type, dict()).items()}

def get_binary_path(properties, type):
	""" Gets binary path """
	return properties.get(type, get_default_value(type))

def check_in_path(path, obj):
	""" Checks input instructions file """ 
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not os.path.exists(path):
		fu.log('Unexisting input instructions file, exiting', out_log, obj.global_log)
		raise SystemExit('Unexisting input instructions file')

	# check syntax for instructions file
	syntax_instructions = True
	err_instructions = ''
	if not 'parm ' in open(path).read():
		syntax_instructions = False
		err_instructions += 'No topology provided'
	if not 'trajin ' in open(path).read():
		syntax_instructions = False
		err_instructions += ' No input trajectory provided'
	if not syntax_instructions:
		fu.log('Incorrect syntax for instructions file: %s, exiting' % err_instructions, out_log, obj.global_log)
		raise SystemExit('Incorrect syntax for instructions file: %s, exiting' % err_instructions)

def get_default_value(key):
	""" Gives default values according to the given key """
	default_values = {
		"start": 1,
		"end": -1,
		"step": 1,
		"snapshot": 1,
		"format": "netcdf",
		"mask": "all-atoms",
		"reference": "first",
		"average": "MyAvg",
		"instructions_file": "instructions.in",
		"cpptraj_path": "cpptraj"
	}

	return default_values[key]

def is_valid_topology(ext):
	""" Checks if trajectory format is compatible with Cpptraj """
	formats = 'top', 'pdb', 'prmtop', 'parmtop'
	return ext in formats

def is_valid_trajectory(traj):
	""" Checks if trajectory format is compatible with Cpptraj """
	formats = 'crd', 'cdf', 'netcdf', 'restart', 'ncrestart', 'restartnc', 'dcd', 'charmm', 'cor', 'pdb', 'mol2', 'trr', 'gro', 'binpos', 'xtc', 'cif', 'arc', 'sqm', 'sdf', 'conflib'
	return traj in formats

def is_valid_reference(ref):
	""" Checks if reference is correct """
	references = 'first', 'average', 'experimental'
	return ref in references

def get_mask_atoms(key):
	""" Gives mask atoms according to the given key """
	masks = {
		"c-alpha": "@CA",
		"backbone": "@C,CA,N,O,C3',O3',C4',C5',O5',P",
		"all-atoms": ":*",
		"heavy-atoms": "!@H*,1H*,2H*,3H*",
		"side-chain": "!@CA,C,N,O,H,HA,C3',O3',C4',C5',O5',P",
		"solute": "!:WAT,HOH,SOL,TIP3,TP3",
		"ions": ":SOD,CLA,Na+,Cl-,NA,CL,K+,K",
		"solvent": ":WAT,HOH,SOL,TIP3,TP3"
	}

	# if key incorrect, return default value and message 
	if key in masks:
		return masks[key], None
	else:
		return masks[get_default_value("mask")], "Mask %s is not compatible, assigned default value: %s" % (key, get_default_value("mask"))

def get_in_parameters(list, obj, type = 'None'):
	""" Return string with input parameters """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)

	# if strip or mask, no mandatory trajin parameters
	if type == 'strip' or type == 'mask':
		start = '' if not 'start' in list else str(list['start'])
		end = '' if not 'end' in list else str(list['end'])
		step = '' if not 'step' in list else str(list['step'])
		if (not start or start == 'None') and (not end or end == 'None') and (not step or step == 'None'):
			return ''
		else:
			if not start: 
				start = str(get_default_value("start"))
				fu.log('No start value provided in configuration file or incorrect format, assigned default value: %s' % get_default_value('start'), out_log, obj.global_log)
			if not end: 
				end = str(get_default_value("end"))
				fu.log('No end value provided in configuration file or incorrect format, assigned default value: %s' % get_default_value('end'), out_log, obj.global_log)
			if not step: 
				step = str(get_default_value("step"))
				fu.log('No step value provided in configuration file or incorrect format, assigned default value: %s' % get_default_value('step'), out_log, obj.global_log)
	else:
		# check if trajin parameters are provided and have correct format
		if(type == 'snapshot'):
			snapshot = str(get_default_value("snapshot")) if not 'snapshot' in list else str(list['snapshot'])
			if not 'snapshot' in list or not list['snapshot'] or not isinstance(list['snapshot'], int):
				snapshot = str(get_default_value("snapshot"))
				fu.log('No snapshot value provided in configuration file or incorrect format, assigned default value: %s' % get_default_value('snapshot'), out_log, obj.global_log)
			start = snapshot
			end = snapshot
			step = '1'
		else:
			start = str(get_default_value("start")) if not 'start' in list else str(list['start'])
			if not 'start' in list or not list['start'] or not isinstance(list['start'], int):
				start = str(get_default_value("start"))
				fu.log('No start value provided in configuration file or incorrect format, assigned default value: %s' % get_default_value('start'), out_log, obj.global_log)
			end = str(get_default_value("end")) if not 'end' in list else str(list['end'])
			if not 'end' in list or not list['end'] or not isinstance(list['end'], int):
				end = str(get_default_value("end"))
				fu.log('No end value provided in configuration file or incorrect format, assigned default value: %s' % get_default_value('end'), out_log, obj.global_log)
			step = str(get_default_value("step")) if not 'step' in list else str(list['step'])
			if not 'step' in list or not list['step'] or not isinstance(list['step'], int):
				step = str(get_default_value("step"))
				fu.log('No step value provided in configuration file or incorrect format, assigned default value: %s' % get_default_value('step'), out_log, obj.global_log)

			# checking start <= end
			if end != '-1' and start > end:
				fu.log('End must be -1 (indicating the end of the trajectory) or more or equal than start. Your values are start: %s, end: %s' % (start, end), out_log, obj.global_log)
				raise SystemExit('End must be -1 (indicating the end of the trajectory) or greater or equal than start. Your values are start: %s, end: %s' % (start, end))

	return start + " " + end + " " + step

def setup_structure(obj):
	""" Sets up the structure """
	instructions_list = []
	mask_atoms = get_mask('heavy-atoms', obj)
	instructions_list.append('center ' + mask_atoms + ' origin')
	instructions_list.append('autoimage')
	instructions_list.append('rms first ' + mask_atoms)
	mask_solvent = get_mask('solvent', obj)
	mask_ions = get_mask('ions', obj)
	instructions_list.append('strip ' + mask_solvent + ',' + mask_ions[1:])

	return instructions_list


def get_negative_mask(key, obj):
	""" Gives the negative mask according to the given key """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	atoms, msg = get_mask_atoms(key)
	if atoms[0] == '!':
		mask = atoms[1:]
	else:
		mask = '!' + atoms
	# if mask incorrect, give message
	if msg: 
		fu.log(msg, out_log, obj.global_log)

	return mask

def get_mask(key, obj):
	""" Gives mask according to the given key """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	mask, msg = get_mask_atoms(key)
	# if mask incorrect, give message
	if msg: 
		fu.log(msg, out_log, obj.global_log)

	return mask

def get_reference(ref, obj, mask, output):
	""" Gives reference instructions according to the given key """
	instructions_list = []

	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)
	if not ref or ref == 'None':
		ref = get_default_value('reference')
		fu.log('No reference provided in configuration file, assigned default value: %s' % get_default_value('reference'), out_log, obj.global_log)

	if not is_valid_reference(ref):
		fu.log('Reference %s is not compatible, assigned default value: %s' % (ref, get_default_value('reference')), out_log, obj.global_log)
		ref = get_default_value('reference')

	if ref == 'first':
		if output: instructions_list.append('rms first out ' + obj.output_cpptraj_path)
		else: instructions_list.append('rms first')

	if ref == 'average':
		instructions_list.append('average crdset ' + get_default_value('average'))
		instructions_list.append('run')
		if output: instructions_list.append('rms ref ' + get_default_value('average') + ' ' + mask + ' out ' + obj.output_cpptraj_path)
		else: instructions_list.append('rms ref ' + get_default_value('average') + ' ' + mask)

	if ref == 'experimental':
		if not obj.input_exp_path:
			fu.log('No experimental structure provided, exiting', out_log, obj.global_log)
			raise SystemExit('input_exp_path is mandatory')
		instructions_list.append('parm ' + obj.input_exp_path + ' noconect [exp]')
		solute, msg = get_mask_atoms('solute')
		instructions_list.append('reference ' + obj.input_exp_path + ' ' + solute + ' parm [exp]')
		backbone, msg = get_mask_atoms('backbone')
		if output: instructions_list.append('rms reference ' + backbone + ' ' + mask + ' out ' + obj.output_cpptraj_path)
		else: instructions_list.append('rms reference ' + backbone + ' ' + mask)


	return instructions_list

def get_out_parameters(list, obj):
	""" Return string with output parameters """
	out_log, err_log = fu.get_logs(path=obj.path, prefix=obj.prefix, step=obj.step, can_write_console=obj.can_write_console_log)

	format = list['format']
	msg = None
	 # check if format provided
	if not format:
		format = get_default_value('format')
		fu.log('No format provided in configuration file, assigned default value: %s' % get_default_value('format'), out_log, obj.global_log)
	# check if valid format
	if not is_valid_trajectory(format):
		fu.log('Format %s is not compatible, assigned default value: %s' % (format, get_default_value('format')), out_log, obj.global_log)
		format = get_default_value('format')

	return format


#!/usr/bin/env python3

"""Module containing the GMX Energy class and the command line interface."""
import argparse
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.command_wrapper import cmd_wrapper
from biobb_analysis.gromacs.common import *


class GMXEnergy():
    """Wrapper of the GROMACS energy (http://manual.gromacs.org/current/onlinehelp/gmx-energy.html) module.

    Args:
        input_energy_path (str): Path to the input EDR file.
        output_xvg_path (str): Path to the XVG output file.
        properties (dic):
            * **xvg** (*str*) - ("none") XVG plot formatting: xmgrace, xmgr, none.
            * **terms** (*list*) - (["Potential"]) Energy terms. Select one or more from: Angle, Proper-Dih., Improper-Dih., LJ-14, Coulomb-14, LJ-(SR), Coulomb-(SR), Coul.-recip., Position-Rest., Potential, Kinetic-En., Total-Energy, Temperature, Pressure,  Constr.-rmsd, Box-X, Box-Y,  Box-Z, Volume, Density, pV, Enthalpy, Vir-XX, Vir-XY, Vir-XZ, Vir-YX, Vir-YY, Vir-YZ, Vir-ZX, Vir-ZY, Vir-ZZ, Pres-XX, Pres-XY, Pres-XZ, Pres-YX, Pres-YY,  Pres-YZ, Pres-ZX, Pres-ZY, Pres-ZZ, #Surf*SurfTen, Box-Vel-XX, Box-Vel-YY, Box-Vel-ZZ, Mu-X, Mu-Y, Mu-Z, T-Protein, T-non-Protein, Lamb-Protein, Lamb-non-Protein
            * **gmx_path** (*str*) - ("gmx") Path to the GROMACS executable binary.
    """

    def __init__(self, input_energy_path, output_xvg_path, properties=None, **kwargs):
        properties = properties or {}

        # Input/Output files
        self.input_energy_path = input_energy_path
        self.output_xvg_path = output_xvg_path

        # Properties specific for BB
        self.instructions_file = get_default_value('instructions_file')
        self.properties = properties

        # Properties common in all GROMACS BB
        self.gmx_path = get_binary_path(properties, 'gmx_path')

        # Properties common in all BB
        self.can_write_console_log = properties.get('can_write_console_log', True)
        self.global_log = properties.get('global_log', None)
        self.prefix = properties.get('prefix', None)
        self.step = properties.get('step', None)
        self.path = properties.get('path', '')

    def check_data_params(self):
        """ Checks all the input/output paths and parameters """
        self.input_energy_path = check_energy_path(self.input_energy_path, self)
        self.output_xvg_path = check_out_xvg_path(self.output_xvg_path, self)
        self.xvg = get_xvg(self.properties, self)
        self.terms = get_terms(self.properties, self)

    def create_instructions_file(self):
        """Creates an input file using the properties file settings"""
        instructions_list = []
        self.instructions_file = fu.create_name(prefix=self.prefix, step=self.step, name=self.instructions_file)

        for t in self.terms:
            instructions_list.append(t)

        # create instructions file
        with open(self.instructions_file, 'w') as mdp:
            for line in instructions_list:
                mdp.write(line.strip() + '\n')

        return self.instructions_file

    def launch(self):
        """Launches the execution of the GROMACS energy module."""
        out_log, err_log = fu.get_logs(path=self.path, prefix=self.prefix, step=self.step, can_write_console=self.can_write_console_log)

        # check input/output paths and parameters
        self.check_data_params()

        # create instructions file
        self.create_instructions_file() 

        cmd = [self.gmx_path, 'energy',
               '-f', self.input_energy_path,
               '-o', self.output_xvg_path,
               '-xvg', self.xvg,
               '<', self.instructions_file]

        returncode = cmd_wrapper.CmdWrapper(cmd, out_log, err_log, self.global_log).launch()
        tmp_files = [self.instructions_file]
        removed_files = [f for f in tmp_files if fu.rm(f)]
        fu.log('Removed: %s' % str(removed_files), out_log, self.global_log)
        return returncode

def main():
    parser = argparse.ArgumentParser(description="Wrapper for the GROMACS energy module.")
    parser.add_argument('--config', required=False)
    parser.add_argument('--system', required=False)
    parser.add_argument('--step', required=False)

    #Specific args of each building block
    parser.add_argument('--input_energy_path', required=True)
    parser.add_argument('--output_xvg_path', required=True)

    args = parser.parse_args()
    check_conf(args.config)
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config, system=args.system).get_prop_dic()
    if args.step:
        properties = properties[args.step]

    #Specific call of each building block
    GMXEnergy(input_energy_path=args.input_energy_path, output_xvg_path=args.output_xvg_path, properties=properties).launch()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""Module containing the Cpptraj Input class and the command line interface."""
import argparse
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.command_wrapper import cmd_wrapper
from biobb_analysis.ambertools.common import *

class CpptrajInput():
    """Wrapper of the Ambertools CpptrajInput module.
    Cpptraj (the successor to ptraj) is the main program in Ambertools for processing coordinate trajectories and data files.
    The parameter names and defaults are the same as
    the ones in the official Cpptraj manual: https://amber-md.github.io/cpptraj/CPPTRAJ.xhtml

    Args:
        input_instructions_path (str): Path of the instructions file.
        properties (dic):
            * **cpptraj_path** (*str*) - ("cpptraj") Path to the cpptraj executable binary.
    """

    def __init__(self, input_instructions_path=None, properties=None, **kwargs):
        properties = properties or {}

        # Properties specific for BB
        self.input_instructions_path = input_instructions_path
        self.input_top_path = kwargs.get('input_top_path')
        self.input_traj_path = kwargs.get('input_traj_path')
        self.output_cpptraj_path = kwargs.get('output_cpptraj_path')
        self.cpptraj_path = properties.get('cpptraj_path', 'cpptraj')

        # Properties common in all BB
        self.can_write_console_log = properties.get('can_write_console_log', True)
        self.global_log = properties.get('global_log', None)
        self.prefix = properties.get('prefix', None)
        self.step = properties.get('step', None)
        self.path = properties.get('path', '')

    def create_instrucions_file(self):
        """ Creates an input file using paths provideed in the configuration file (only used for test purposes) """
        instructions_list = []
        output_instructions_path = fu.create_name(prefix=self.prefix, step=self.step, name=get_default_value("instructions_file"))

        instructions_list.append('parm ' + self.input_top_path)
        instructions_list.append('trajin ' + self.input_traj_path)
        instructions_list.append('trajout ' + self.output_cpptraj_path + ' ' + get_default_value("format"))

        with open(output_instructions_path, 'w') as mdp:
            for line in instructions_list:
                mdp.write(line.strip() + '\n')

        return output_instructions_path

    def launch(self):
        """Launches the execution of the Ambertools cpptraj module."""
        out_log, err_log = fu.get_logs(path=self.path, prefix=self.prefix, step=self.step, can_write_console=self.can_write_console_log)

        output_instructions_path = self.create_instrucions_file() if not self.input_instructions_path else self.input_instructions_path
        check_in_path(output_instructions_path, self)

        # run command line
        cmd = [self.cpptraj_path, '-i', output_instructions_path]

        returncode = cmd_wrapper.CmdWrapper(cmd, out_log, err_log, self.global_log).launch()
        return returncode

def main():
    parser = argparse.ArgumentParser(description="Wrapper for the Ambertools cpptraj input module.")
    parser.add_argument('--config', required=False, help='Configuration file')

    parser.add_argument('--system', required=False)
    parser.add_argument('--step', required=False)

    parser.add_argument('--input_instructions_path', required=True, help='Path of the instructions file.')

    args = parser.parse_args()
    check_conf(args.config)
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config, system=args.system).get_prop_dic()
    if args.step:
        properties = properties[args.step]

    #Specific call of each building block
    CpptrajInput(input_instructions_path=args.input_instructions_path, properties=properties).launch()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""Module containing the Cpptraj Image class and the command line interface."""
import argparse
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.command_wrapper import cmd_wrapper
from biobb_analysis.ambertools.common import *


class Image():
    """Wrapper of the Ambertools Cpptraj Image module.
    Cpptraj (the successor to ptraj) is the main program in Ambertools for processing coordinate trajectories and data files.
    The parameter names and defaults are the same as
    the ones in the official Cpptraj manual: https://amber-md.github.io/cpptraj/CPPTRAJ.xhtml

    Args:
        input_top_path (str): Path to the input structure or topology file. Accepted formats: top, pdb, prmtop, parmtop.
        input_traj_path (str): Path to the input trajectory to be processed. Accepted formats: crd, cdf, netcdf, restart, ncrestart, restartnc, dcd, charmm, cor, pdb, mol2, trr, gro, binpos, xtc, cif, arc, sqm, sdf, conflib.
        output_cpptraj_path (str): Path to the output processed trajectory.
        properties (dic):
            * **in_parameters** (*dict*) - (None) Parameters for input trajectory. Accepted parameters:
                * **start** (*int*) - (1) Starting frame for slicing
                * **end** (*int*) - (-1) Ending frame for slicing
                * **step** (*int*) - (1) Step for slicing
                * **mask** (*string*) - ("all-atoms") Mask definition. Values: c-alpha, backbone, all-atoms, heavy-atoms, side-chain, solute, ions, solvent.
            * **out_parameters** (*dict*) - (None) Parameters for output trajectory.
                * **format** (*str*) - ("netcdf") Output trajectory format. Values: crd, cdf, netcdf, restart, ncrestart, restartnc, dcd, charmm, cor, pdb, mol2, trr, gro, binpos, xtc, cif, arc, sqm, sdf, conflib.
            * **cpptraj_path** (*str*) - ("cpptraj") Path to the cpptraj executable binary.
    """

    def __init__(self, input_top_path, input_traj_path,
                 output_cpptraj_path, properties=None, **kwargs):
        properties = properties or {}

        # Input/Output files
        self.input_top_path = input_top_path
        self.input_traj_path = input_traj_path
        self.output_cpptraj_path = output_cpptraj_path

        # Properties specific for BB
        self.instructions_file = get_default_value('instructions_file')
        self.properties = properties
        self.cpptraj_path = get_binary_path(properties, 'cpptraj_path')

        # Properties common in all BB
        self.can_write_console_log = properties.get('can_write_console_log', True)
        self.global_log = properties.get('global_log', None)
        self.prefix = properties.get('prefix', None)
        self.step = properties.get('step', None)
        self.path = properties.get('path', '')

    def check_data_params(self):
        """ Checks all the input/output paths and parameters """
        self.input_top_path = check_top_path(self.input_top_path, self)
        self.input_traj_path = check_traj_path(self.input_traj_path, self)
        self.output_cpptraj_path = check_out_path(self.output_cpptraj_path, self)
        self.in_parameters = get_parameters(self.properties, 'in_parameters', self)
        self.out_parameters = get_parameters(self.properties, 'out_parameters', self)

    def create_instructions_file(self):
        """Creates an input file using the properties file settings"""
        instructions_list = []
        self.instructions_file = fu.create_name(prefix=self.prefix, step=self.step, name=self.instructions_file)

        # parm
        instructions_list.append('parm ' + self.input_top_path)

        # trajin
        in_params = get_in_parameters(self.in_parameters, self)
        instructions_list.append('trajin ' + self.input_traj_path + ' ' + in_params)

        # image
        mask_atoms = get_mask('heavy-atoms', self)
        instructions_list.append('center ' + mask_atoms + ' origin')
        instructions_list.append('autoimage')
        instructions_list.append('rms first ' + mask_atoms)

        # mask
        mask = self.in_parameters.get('mask', '')
        if mask:
            strip_mask = get_negative_mask(mask, self)
            instructions_list.append('strip ' + strip_mask)

        # trajout
        out_params = get_out_parameters(self.out_parameters, self)
        instructions_list.append('trajout ' + self.output_cpptraj_path + ' ' + out_params)

        # create .in file
        with open(self.instructions_file, 'w') as mdp:
            for line in instructions_list:
                mdp.write(line.strip() + '\n')

        return self.instructions_file

    def launch(self):
        """Launches the execution of the Ambertools cpptraj module."""
        out_log, err_log = fu.get_logs(path=self.path, prefix=self.prefix, step=self.step, can_write_console=self.can_write_console_log)

        # check input/output paths and parameters
        self.check_data_params()

        # create instructions file
        self.create_instructions_file() 

        # run command line
        cmd = [self.cpptraj_path, '-i', self.instructions_file]

        returncode = cmd_wrapper.CmdWrapper(cmd, out_log, err_log, self.global_log).launch()
        tmp_files = [self.instructions_file]
        removed_files = [f for f in tmp_files if fu.rm(f)]
        fu.log('Removed: %s' % str(removed_files), out_log, self.global_log)
        return returncode

def main():
    parser = argparse.ArgumentParser(description="Wrapper for the Ambertools cpptraj image module.")
    parser.add_argument('--config', required=True, help='Configuration file')
    parser.add_argument('--system', required=False)
    parser.add_argument('--step', required=False)

    # Specific args of each building block
    parser.add_argument('--input_top_path', required=True, help='Path to the input Amber structure or topology file.')
    parser.add_argument('--input_traj_path', required=True, help='Path to the input Amber trajectory to be processed.')
    parser.add_argument('--output_cpptraj_path', required=True, help='Path to the output processed Amber trajectory or to the output dat file containing the analysis results.')

    args = parser.parse_args()
    check_conf(args.config)
    args.config = args.config or "{}"
    properties = settings.ConfReader(config=args.config, system=args.system).get_prop_dic()
    if args.step:
        properties = properties[args.step]

    # Specific call of each building block
    Image(input_top_path=args.input_top_path, input_traj_path=args.input_traj_path, output_cpptraj_path=args.output_cpptraj_path, properties=properties).launch()

if __name__ == '__main__':
    main()